#pragma once
#include <iostream>
#include <iomanip>
#include <fstream>
#include <ctime>
#include <vector>
#include <algorithm>
#include <iterator>
#include <string>
#include <cmath>

#define N 100

bool createData(const std::string& fileName) {
	srand(time(nullptr));
	std::ofstream fout(fileName, std::ios::out | std::ios::trunc);
	if (!fout) {
		throw std::exception("Wrong (file name is not valid)!");
		return false;
	}
	for (int i = 0; i < N; ++i) {
		fout << rand() % 101 - 50 << "\n";
	}
	fout.close();
	return true;
}


template<class container>
bool createDataGenerate(const std::string& fileName) {
	srand(time(nullptr));
	
	using type = container::value_type; //!!! Где typename ?
	
	std::ofstream fout(fileName, std::ios::out | std::ios::trunc);
	
	container<type> values(N);//!!! container - это инстанцированный тип, а не шаблон. Откуда параметр type взялся?
	
	if (!fout) {
		throw std::exception("Wrong (file name is not valid)!");
		return false;
	}
	std::generate(values.begin(), values.end(), []() { return (std::rand() % 101 - 50); });
	std::copy(values.begin(), values.end(), std::ostream_iterator<int>(fout, "\n"));
	fout.close();
	return true;
}

template<class container>
	container& loadData (const std::string& file)
{
	using type = container::value_type; //!!! Где typename ?
	
	container<type> result;//!!! container - это инстанцированный тип, а не шаблон. Откуда параметр type взялся?
	
	std::ifstream fin(file);

	if (!fin.is_open())
	{
		throw std::exception("Wrong (file name is not valid)!");
	}
	int readBuf;
	while (fin >> readBuf)
	{
		result.push_back(readBuf);
	}
	fin.close();

	return result;
};

	template<class container>
	container& loadDataIterator(const std::string& fileName) {
		using type = container::value_type;  //!!! Где typename ?
		
		container<type> result(N); //!!! container - это инстанцированный тип, а не шаблон. Откуда параметр type взялся?
		
		std::ifstream fin(fileName);
		if (fin) {
			std::copy(std::istream_iterator<type>(fin), std::istream_iterator<type>(), result.begin());
		}
		return result;
}
	template<class container>
bool Modify1(container &cont) {
	if (cont.size() == 0) {
		return false;
	}
	
	using type = container::value_type; //!!! Где typename ?
	
	type firstUneven = type();
	type sum = type();
	for (const auto& i : cont) {
		if (i % 2 != 0) {
			firstUneven = i;
			break;
		}
	}
	type sum2 = type();
	for (auto& i : cont) {
		i += firstUneven;
		sum+=i;
		sum2 += std::abs(i);
	}

	cont.push_back(sum);
	cont.push_back(sum2/100);
	return true;
}

template<class container>
bool Modify2(typename container::iterator beg, typename container::iterator  end) {
	if (end == beg) {
		return false;
	}
	
	using type = container::value_type;  //!!! Где typename ?
	
	type firstUneven = type();
	type sum = type();
	for (const auto& iter = beg; iter != end; ++iter) {
		if ((*iter) % 2 != 0) {
			firstUneven = *iter;
			break;
		}
	}

	for (auto& iter = beg; iter != end; ++iter) {
		*iter += firstUneven;
	}

	return true;
}

template<class container>
bool Modify3(container & cont) {
	if (cont.end() == cont.begin()) {
		return false;
	}
	using type = container::value_type; //!!! Где typename ?
	
	auto firstUneven = std::find_if(cont.begin(), cont.end(), [](type i) { return i % 2 != 0; });
	if (!firstUneven) *firstUneven = type();
	std::for_each(cont.begin(), cont.end(), [firstUneven](auto& obj) { obj += *firstUneven; });

	return true;
}

template<class container>
bool Modify4(const std::string& fileName) {
	using type = container::value_type;  //!!! Где typename ?
	
	container buffer(N);
	std::ifstream fin(fileName);
	if (!fin) {
	throw std::exception("Wrong (file name is not valid)!");
	}
	std::copy(std::istream_iterator<type>(fin), std::istream_iterator<type>(), buffer.begin());
	auto firstUneven = std::find_if(buffer.begin(), buffer.end(), [](type i) { return i % 2 != 0; });
	std::ofstream fout("res.txt");
	std::transform(buffer.begin(), buffer.end(), std::ostream_iterator<type>(fout, "\n"),
		[ firstUneven](auto& obj) { return obj += *firstUneven; });

	return true;
}

template<class container>
bool outputResult(const std::string& fileName, const container& cont) {
	std::ofstream fout(fileName, std::ios::out | std::ios::trunc);
	if (!fout) {
		throw std::exception("Wrong (file name is not valid)!");
		return false;
	}
	for (int i = 0; i < cont.size(); ++i) {
		fout << cont[i] <<"\n";
	}
	fout.close();
	return true;
}

template<class container>
bool outputResultCopy(const std::string& fileName, const container& cont) {
	srand(time(nullptr));
	std::ofstream fout(fileName, std::ios::out | std::ios::trunc);
	if (!fout) {
		throw std::exception("Wrong (file name is not valid)!");
		return false;
	}
	
	using type = container::value_type;  //!!! Где typename ?
	
	std::copy(cont.begin(), cont.end(), std::ostream_iterator<type>(fout, "\n"));
	fout.close();
	return true;
}
