#pragma once
#include <fstream>
#include <set>
#include <iostream>
#include <string.h>

class Person {
private:
	struct Data {
		char full_name[256];
		char enrollment_date[256];
		int personnel_number;
		int department_number;
		int salary;
		int bonus_percentage;
		int income_tax;
		int number_of_days_worked;
		

		Data() :
			full_name(""),
			enrollment_date(""),
			personnel_number(0),
			department_number(0),
			salary(0),
			bonus_percentage(0),
			income_tax(0),
			number_of_days_worked(0)
		{}

		Data(char* _full_name,
			char* _enrollment_date,
			int _personnel_number,
			int _department_number,
			int _salary,
			int _bonus_percentage,
			int _income_tax,
			int _number_of_days_worked) :
			personnel_number(_personnel_number),
			department_number(_department_number),
			salary(_salary),
			bonus_percentage(_bonus_percentage),
			income_tax(_income_tax),
			number_of_days_worked(_number_of_days_worked)
		{
			strncpy_s(full_name, _full_name, 256);
			strncpy_s(enrollment_date, _enrollment_date, 256);
		}


		bool operator == (const Data& data) const {
			return (strcmp(full_name, data.full_name)==0 &&
				strcmp(enrollment_date,data.enrollment_date)==0 &&
				personnel_number == data.personnel_number &&
				department_number == data.department_number &&
				salary == data.salary &&
				bonus_percentage == data.bonus_percentage &&
				income_tax == data.income_tax &&
				number_of_days_worked == data.number_of_days_worked);
		}
		bool operator != (const Data& data) {
			return !(*this == data);
		}

		friend bool operator < (const Data& data, const Data& data2) {
			return data.number_of_days_worked < data2.number_of_days_worked;
		}

		bool in(const std::multiset<Data> cont) const{
			for (auto i = cont.begin(); i != cont.end(); ++i) {
				if (*this == *i)
					return true;
			}
			return false;
		}

	};
	std::multiset<Data> cont;

public:

	Person() : cont(std::multiset<Data>()) {}

	Person(std::multiset<Data> cont) : cont(cont) {}

	bool operator == (const Person&);

	bool operator != (const Person& data) {
		return !(*this == data);
	}

	friend std::ostream& operator << (std::ostream&, const Person&);
	friend std::istream& operator >> (std::istream&, Person&);

	void add(std::ofstream&);
	void read(std::ifstream&);
	static void create_file(std::ofstream&);

	void delete_data(std::ofstream&, std::ifstream&);
	void replacement(std::ofstream&, std::ifstream&, Person&);
	void convert_to_txt(std::ofstream& file_write, std::ifstream& file_read);
	void find_full_name(const std::string& p, std::ofstream& file);
	void find_department_number(int p, std::ofstream& file);
	void find_enrollment_date(const std::string& p, std::ofstream& file);
};
